@@ -138,13 +138,25 @@ champlain_network_map_source_set_property (GObject *object,
         priv->offline = g_value_get_boolean (value);
         break;
       case PROP_PROXY_URI:
-        g_free (priv->proxy_uri);
+        {
+          SoupURI *uri = NULL;
 
-        priv->proxy_uri = g_value_dup_string (value);
-        if (soup_session)
-          g_object_set (G_OBJECT (soup_session), "proxy-uri",
-              soup_uri_new (priv->proxy_uri), NULL);
-        break;
+          g_free (priv->proxy_uri);
+
+          priv->proxy_uri = g_value_dup_string (value);
+          if (priv->proxy_uri)
+            uri = soup_uri_new (priv->proxy_uri);
+
+          if (soup_session)
+            g_object_set (G_OBJECT (soup_session),
+                "proxy-uri", uri,
+                NULL);
+
+          if (uri)
+            g_object_unref (uri);
+
+          break;
+        }
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
     }
@@ -373,43 +385,55 @@ get_filename (ChamplainNetworkMapSource *source,
              champlain_tile_get_x (tile), champlain_tile_get_y (tile));
 }
 
+static ClutterActor *error_actor = NULL;
+
 static void
 create_error_tile (ChamplainTile* tile)
 {
-  ClutterActor *actor;
-  cairo_t *cr;
-  cairo_pattern_t *pat;
-  guint size;
+  ClutterActor *clone;
 
-  size = champlain_tile_get_size (tile);
-  actor = clutter_cairo_texture_new (size, size);
-  cr = clutter_cairo_texture_create (CLUTTER_CAIRO_TEXTURE (actor));
+  if (!error_actor)
+    {
+      guint size;
+      ClutterActor *actor;
+      cairo_t *cr;
+      cairo_pattern_t *pat;
+      ClutterActor *stage;
 
-  /* draw a linear gray to white pattern */
-  pat = cairo_pattern_create_linear (size / 2.0, 0.0,  size, size / 2.0);
-  cairo_pattern_add_color_stop_rgb (pat, 0, 0.686, 0.686, 0.686);
-  cairo_pattern_add_color_stop_rgb (pat, 1, 0.925, 0.925, 0.925);
-  cairo_set_source (cr, pat);
-  cairo_rectangle (cr, 0, 0, size, size);
-  cairo_fill (cr);
+      size = champlain_tile_get_size (tile);
+      actor = clutter_cairo_texture_new (size, size);
+      cr = clutter_cairo_texture_create (CLUTTER_CAIRO_TEXTURE (actor));
+      stage = clutter_stage_get_default ();
 
-  cairo_pattern_destroy (pat);
+      /* draw a linear gray to white pattern */
+      pat = cairo_pattern_create_linear (size / 2.0, 0.0,  size, size / 2.0);
+      cairo_pattern_add_color_stop_rgb (pat, 0, 0.686, 0.686, 0.686);
+      cairo_pattern_add_color_stop_rgb (pat, 1, 0.925, 0.925, 0.925);
+      cairo_set_source (cr, pat);
+      cairo_rectangle (cr, 0, 0, size, size);
+      cairo_fill (cr);
 
-  /* draw the red cross */
-  cairo_set_source_rgb (cr, 0.424, 0.078, 0.078);
-  cairo_set_line_width (cr, 14.0);
-  cairo_set_line_cap (cr, CAIRO_LINE_CAP_ROUND);
-  cairo_move_to (cr, 24, 24);
-  cairo_line_to (cr, 50, 50);
-  cairo_move_to (cr, 50, 24);
-  cairo_line_to (cr, 24, 50);
-  cairo_stroke (cr);
+      cairo_pattern_destroy (pat);
 
-  cairo_destroy (cr);
+      /* draw the red cross */
+      cairo_set_source_rgb (cr, 0.424, 0.078, 0.078);
+      cairo_set_line_width (cr, 14.0);
+      cairo_set_line_cap (cr, CAIRO_LINE_CAP_ROUND);
+      cairo_move_to (cr, 24, 24);
+      cairo_line_to (cr, 50, 50);
+      cairo_move_to (cr, 50, 24);
+      cairo_line_to (cr, 24, 50);
+      cairo_stroke (cr);
 
-  champlain_tile_set_content (tile, actor, TRUE);
-  clutter_actor_show (actor);
+      cairo_destroy (cr);
 
+      clutter_actor_show (actor);
+      clutter_container_add_actor (CLUTTER_CONTAINER (stage), actor);
+      error_actor = actor;
+    }
+
+  clone = clutter_clone_new (error_actor);
+  champlain_tile_set_content (tile, clone, TRUE);
   champlain_tile_set_state (tile, CHAMPLAIN_STATE_DONE);
 }
 
@@ -611,8 +635,13 @@ fill_tile (ChamplainMapSource *map_source,
 
       if (!soup_session)
         {
-          soup_session = soup_session_async_new_with_options ("proxy-uri",
-              soup_uri_new (priv->proxy_uri),
+          SoupURI *uri = NULL;
+
+          if (priv->proxy_uri)
+            uri = soup_uri_new (priv->proxy_uri);
+
+          soup_session = soup_session_async_new_with_options (
+              "proxy-uri", uri,
 #ifdef HAVE_LIBSOUP_GNOME
               SOUP_SESSION_ADD_FEATURE_BY_TYPE, SOUP_TYPE_PROXY_RESOLVER_GNOME,
 #endif
@@ -622,6 +651,9 @@ fill_tile (ChamplainMapSource *map_source,
               "max-conns-per-host", 2, NULL); // This is as required by OSM
           g_object_add_weak_pointer (G_OBJECT (soup_session),
               (gpointer *) &soup_session);
+
+          if (uri)
+            g_object_unref (uri);
         }
 
       uri = champlain_network_map_source_get_tile_uri (source,
